Package: flowCore
Title: flowCore: Basic structures for flow cytometry data
Version: 2.25.0
Authors@R: c(person("B","Ellis", role = "aut"),
							 person("Perry", "Haaland", role = "aut"),
							 person("Florian", "Hahne", role = "aut"),
							 person("Nolwenn", "Le Meur", role = "aut"),
							 person("Nishant", "Gopalakrishnan", role = "aut"),
							 person("Josef", "Spidlen", role = "aut"),
							 person("Mike", "Jiang", role = c("aut","cre"), email = "mike@ozette.com"),
							 person("Greg", "Finak", role = "aut"),
							 person("Samuel","Granjeaud", role = "ctb"))
Maintainer: Mike Jiang <mike@ozette.com>
Description: Provides S4 data structures and basic functions to deal with flow
    cytometry data.
Depends: R (>= 3.5.0)
Imports: Biobase, BiocGenerics (>= 0.29.2), grDevices, graphics,
        methods, stats, utils, stats4, Rcpp, matrixStats, cytolib (>=
        2.13.1), S4Vectors
Suggests: Rgraphviz, flowViz, flowStats (>= 3.43.4), testthat,
        flowWorkspace, flowWorkspaceData, openCyto, knitr, ggcyto,
        gridExtra
Collate: AllGenerics.R AllClasses.R flowFrame-accessors.R
        flowSet-accessors.R transform_gate-methods.R coerce.R
        logicalFilterResult-accessors.R summarizeFilter-methods.R
        filterSummary-accessors.R manyFilterResult-accessors.R
        summary-methods.R multipleFilterResult-accessors.R on-methods.R
        transformList-accessors.R identifier-methods.R
        parameters-methods.R initialize-methods.R
        filterResult-accessors.R in-methods.R rectangleGate-accessors.R
        filterResultList-accessors.R IO.R show-methods.R
        length-methods.R names-methods.R split-methods.R eval-methods.R
        gatingML.R FCSTransTransform.R median-logicle-transform.R
        utils.R flowCore.R GvHD.R CytoExploreR_wrappers.R cpp11.R
License: Artistic-2.0
biocViews: ImmunoOncology, Infrastructure, FlowCytometry,
        CellBasedAssays
LinkingTo: cpp11, BH(>= 1.81.0.0), cytolib, RProtoBufLib
VignetteBuilder: knitr
SystemRequirements: GNU make, C++17
RoxygenNote: 7.2.3
Encoding: UTF-8
git_url: https://git.bioconductor.org/packages/flowCore
git_branch: devel
git_last_commit: 166a86e
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
NeedsCompilation: yes
Packaged: 2026-04-28 22:40:30 UTC; biocbuild
Author: B Ellis [aut],
  Perry Haaland [aut],
  Florian Hahne [aut],
  Nolwenn Le Meur [aut],
  Nishant Gopalakrishnan [aut],
  Josef Spidlen [aut],
  Mike Jiang [aut, cre],
  Greg Finak [aut],
  Samuel Granjeaud [ctb]
