# Generated by roxygen2: do not edit by hand

S3method("$",flowFrame)
S3method(as.data.frame,manyFilterResult)
S3method(estimateLogicle,flowFrame)
S3method(inverseLogicleTransform,default)
S3method(inverseLogicleTransform,transform)
S3method(inverseLogicleTransform,transformList)
S3method(rotate_gate,default)
S3method(rotate_gate,ellipsoidGate)
S3method(rotate_gate,polygonGate)
S3method(rotate_gate,quadGate)
S3method(rotate_gate,rectangleGate)
S3method(scale_gate,default)
S3method(scale_gate,ellipsoidGate)
S3method(scale_gate,polygonGate)
S3method(scale_gate,quadGate)
S3method(scale_gate,rectangleGate)
S3method(shift_gate,default)
S3method(shift_gate,ellipsoidGate)
S3method(shift_gate,polygonGate)
S3method(shift_gate,quadGate)
S3method(shift_gate,rectangleGate)
S3method(transform_gate,default)
export("%&%")
export("%in%")
export("%on%")
export("%subset%")
export("colnames<-")
export("filterDetails<-")
export("identifier<-")
export("keyword<-")
export("markernames<-")
export("parameters<-")
export(CytoExploreR_.estimateLogicle)
export(EHtrans)
export(FCSTransTransform)
export(Subset)
export(arcsinhTransform)
export(asinht)
export(asinhtGml2)
export(biexponentialTransform)
export(boundaryFilter)
export(char2ExpressionFilter)
export(cleanup)
export(collapse_desc)
export(compensate)
export(compensatedParameter)
export(compensation)
export(decompensate)
export(dg1polynomial)
export(each_col)
export(each_row)
export(ellipsoidGate)
export(estimateLogicle)
export(estimateMedianLogicle)
export(exponential)
export(expressionFilter)
export(filter)
export(filterDetails)
export(filterList)
export(filterReference)
export(filters)
export(filtersList)
export(flowFrame)
export(flowSet)
export(flowSet_to_list)
export(fr_append_cols)
export(fsApply)
export(getChannelMarker)
export(getIndexSort)
export(head)
export(hyperlog)
export(hyperlogtGml2)
export(identifier)
export(inverseLogicleTransform)
export(invsplitscale)
export(isFCSfile)
export(keyword)
export(kmeansFilter)
export(linearTransform)
export(lintGml2)
export(lnTransform)
export(logTransform)
export(logarithm)
export(logicleTransform)
export(logicletGml2)
export(logtGml2)
export(manyFilterResult)
export(markernames)
export(multiRangeGate)
export(normalization)
export(normalize)
export(parameters)
export(polygonGate)
export(polytopeGate)
export(print)
export(quadGate)
export(quadratic)
export(quadraticTransform)
export(ratio)
export(ratiotGml2)
export(read.FCS)
export(read.FCSheader)
export(read.flowSet)
export(rectangleGate)
export(rotate_gate)
export(sampleFilter)
export(scaleTransform)
export(scale_gate)
export(shift_gate)
export(sinht)
export(spillover)
export(split)
export(splitScaleTransform)
export(splitscale)
export(squareroot)
export(summarizeFilter)
export(tail)
export(timeFilter)
export(transform)
export(transformList)
export(transformReference)
export(transform_gate)
export(truncateTransform)
export(unitytransform)
export(write.FCS)
export(write.flowSet)
exportClasses(EHtrans)
exportClasses(asinht)
exportClasses(asinhtGml2)
exportClasses(boundaryFilter)
exportClasses(compensatedParameter)
exportClasses(compensation)
exportClasses(complementFilter)
exportClasses(concreteFilter)
exportClasses(dg1polynomial)
exportClasses(ellipsoidGate)
exportClasses(exponential)
exportClasses(expressionFilter)
exportClasses(filter)
exportClasses(filterList)
exportClasses(filterReference)
exportClasses(filterResult)
exportClasses(filterResultList)
exportClasses(filterSummary)
exportClasses(filterSummaryList)
exportClasses(filters)
exportClasses(filtersList)
exportClasses(flowFrame)
exportClasses(flowSet)
exportClasses(hyperlog)
exportClasses(hyperlogtGml2)
exportClasses(intersectFilter)
exportClasses(invsplitscale)
exportClasses(kmeansFilter)
exportClasses(lintGml2)
exportClasses(logarithm)
exportClasses(logicalFilterResult)
exportClasses(logicletGml2)
exportClasses(logtGml2)
exportClasses(manyFilterResult)
exportClasses(multiRangeGate)
exportClasses(normalization)
exportClasses(parameterFilter)
exportClasses(parameterTransform)
exportClasses(parameters)
exportClasses(polygonGate)
exportClasses(polytopeGate)
exportClasses(quadGate)
exportClasses(quadratic)
exportClasses(randomFilterResult)
exportClasses(ratio)
exportClasses(ratiotGml2)
exportClasses(rectangleGate)
exportClasses(sampleFilter)
exportClasses(setOperationFilter)
exportClasses(sinht)
exportClasses(splitscale)
exportClasses(squareroot)
exportClasses(subsetFilter)
exportClasses(timeFilter)
exportClasses(transform)
exportClasses(transformFilter)
exportClasses(transformList)
exportClasses(transformMap)
exportClasses(transformReference)
exportClasses(unionFilter)
exportClasses(unitytransform)
exportMethods("!")
exportMethods("$")
exportMethods("%&%")
exportMethods("%in%")
exportMethods("%on%")
exportMethods("%subset%")
exportMethods("&")
exportMethods("*")
exportMethods("<")
exportMethods("<=")
exportMethods("==")
exportMethods(">")
exportMethods(">=")
exportMethods("[")
exportMethods("[[")
exportMethods("colnames<-")
exportMethods("description<-")
exportMethods("exprs<-")
exportMethods("filterDetails<-")
exportMethods("identifier<-")
exportMethods("markernames<-")
exportMethods("pData<-")
exportMethods("phenoData<-")
exportMethods("sampleNames<-")
exportMethods("varLabels<-")
exportMethods("varMetadata<-")
exportMethods("|")
exportMethods(Subset)
exportMethods(c)
exportMethods(cbind2)
exportMethods(colnames)
exportMethods(compensate)
exportMethods(decompensate)
exportMethods(description)
exportMethods(dim)
exportMethods(each_col)
exportMethods(each_row)
exportMethods(eval)
exportMethods(exprs)
exportMethods(featureNames)
exportMethods(filter)
exportMethods(filterDetails)
exportMethods(filterReference)
exportMethods(fsApply)
exportMethods(getIndexSort)
exportMethods(head)
exportMethods(identifier)
exportMethods(initialize)
exportMethods(keyword)
exportMethods(length)
exportMethods(markernames)
exportMethods(names)
exportMethods(ncol)
exportMethods(normalize)
exportMethods(nrow)
exportMethods(pData)
exportMethods(parameters)
exportMethods(phenoData)
exportMethods(plot)
exportMethods(print)
exportMethods(range)
exportMethods(rbind2)
exportMethods(sampleNames)
exportMethods(show)
exportMethods(spillover)
exportMethods(split)
exportMethods(summarizeFilter)
exportMethods(summary)
exportMethods(tail)
exportMethods(toTable)
exportMethods(transform)
exportMethods(varLabels)
exportMethods(varMetadata)
import(methods)
importClassesFrom(Biobase,AnnotatedDataFrame)
importFrom(Biobase,listLen)
importFrom(Biobase,read.AnnotatedDataFrame)
importFrom(BiocGenerics,colnames)
importFrom(BiocGenerics,eval)
importFrom(BiocGenerics,get)
importFrom(BiocGenerics,mget)
importFrom(BiocGenerics,ncol)
importFrom(BiocGenerics,nrow)
importFrom(BiocGenerics,sort)
importFrom(BiocGenerics,toTable)
importFrom(S4Vectors,DataFrame)
importFrom(S4Vectors,coolcat)
importFrom(cytolib,cytolib_LdFlags)
importFrom(grDevices,devAskNewPage)
importFrom(graphics,abline)
importFrom(graphics,hist)
importFrom(graphics,text)
importFrom(matrixStats,colMaxs)
importFrom(matrixStats,colMins)
importFrom(stats,density)
importFrom(stats,kmeans)
importFrom(stats,mad)
importFrom(stats,median)
importFrom(stats,na.omit)
importFrom(stats,qnorm)
importFrom(stats,quantile)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,uniroot)
importFrom(stats,var)
importFrom(stats4,plot)
importFrom(stats4,summary)
importFrom(utils,data)
importFrom(utils,find)
importFrom(utils,head)
importFrom(utils,read.csv)
importFrom(utils,tail)
importFrom(utils,write.table)
importMethodsFrom(Biobase,"description<-")
importMethodsFrom(Biobase,"exprs<-")
importMethodsFrom(Biobase,"pData<-")
importMethodsFrom(Biobase,"phenoData<-")
importMethodsFrom(Biobase,"sampleNames<-")
importMethodsFrom(Biobase,"varLabels<-")
importMethodsFrom(Biobase,"varMetadata<-")
importMethodsFrom(Biobase,description)
importMethodsFrom(Biobase,exprs)
importMethodsFrom(Biobase,featureNames)
importMethodsFrom(Biobase,pData)
importMethodsFrom(Biobase,phenoData)
importMethodsFrom(Biobase,sampleNames)
importMethodsFrom(Biobase,varLabels)
importMethodsFrom(Biobase,varMetadata)
useDynLib(flowCore, .registration=TRUE)
