#' CytoExploreR exports
#' 
#' Exported wrappers of internal functions for use by CytoExploreR
#' 
#' @name CytoExploreR_exports
NULL

#' @keywords internal
#' @name CytoExploreR_.estimateLogicle
#' @rdname CytoExploreR_exports
#' @export
CytoExploreR_.estimateLogicle <- function(x, channels, ...){
  .estimateLogicle(x, channels, ...)
}