# Generated by cpp11: do not edit by hand

biexponential_transform <- function(input, A, B, C, D, F, W, tol, maxIt) {
  .Call(`_flowCore_biexponential_transform`, input, A, B, C, D, F, W, tol, maxIt)
}

convertRawBytes <- function(bytes, isInt, colSize, ncol, isBigEndian) {
  .Call(`_flowCore_convertRawBytes`, bytes, isInt, colSize, ncol, isBigEndian)
}

string_to_spill <- function(key) {
  .Call(`_flowCore_string_to_spill`, key)
}

spill_to_string <- function(rmat, markers) {
  .Call(`_flowCore_spill_to_string`, rmat, markers)
}

fcsTextParse <- function(txt, emptyValue) {
  .Call(`_flowCore_fcsTextParse`, txt, emptyValue)
}

hyperlog_transform <- function(input, T, W, M, A, isInverse) {
  .Call(`_flowCore_hyperlog_transform`, input, T, W, M, A, isInverse)
}

inPolygon <- function(data, vertices) {
  .Call(`_flowCore_inPolygon`, data, vertices)
}

inPolytope <- function(data, A, b) {
  .Call(`_flowCore_inPolytope`, data, A, b)
}

logicle_transform <- function(input, T, W, M, A, isInverse) {
  .Call(`_flowCore_logicle_transform`, input, T, W, M, A, isInverse)
}

poly_centroid <- function(verts) {
  .Call(`_flowCore_poly_centroid`, verts)
}

sortBytes <- function(bytes, byte_order) {
  .Call(`_flowCore_sortBytes`, bytes, byte_order)
}

uint2double <- function(input, isBigEndian) {
  .Call(`_flowCore_uint2double`, input, isBigEndian)
}
